<?php

namespace Drupal\dev_modules;

/**
 * Common code for Drush 9 and older.
 */
abstract class DevModules implements DevModulesInterface {

  protected $enable = TRUE;
  protected $developmentModules;
  protected $productionModules;
  protected $enableModules;
  protected $disableModules;

  /**
   *
   */
  public function __construct() {
    $this->developmentModules = $this->developmentDefaults();
    $this->productionModules = $this->productionDefaults();
    $this->alterModuleLists();
  }

  /**
   * @return $this
   */
  protected abstract function alterModuleLists();

  /**
   * @param string $module
   *
   * @return bool
   */
  protected abstract function moduleExists($module);

  /**
   * @return $this
   */
  protected abstract function enableModules();

  /**
   * @return $this
   */
  protected abstract function disableModules();

  /**
   * @inheritdoc
   */
  public function setEnableMode($flag) {
    $this->enable = $this->flag($flag);
    return $this;
  }

  protected function prepareEnableModule($modules) {
    $module_data = system_rebuild_module_data();
    $list = array();
    foreach ($modules as $module) {
      if (isset($module_data[$module]) && !$this->moduleExists($module)) {
        $list[] = $module;
      }
    }
    return $list;
  }

  protected function prepareDisableModule($modules) {
    $list = array();
    foreach ($modules as $module) {
      if ($this->moduleExists($module)) {
        $list[] = $module;
      }
    }
    return $list;
  }

  /**
   * @inheritDoc
   */
  public function prepareModules() {
    if ($this->enable) {
      $this->enableModules = $this->prepareEnableModule($this->developmentModules);
      $this->disableModules = $this->prepareDisableModule($this->productionModules);
    }
    else {
      $this->enableModules = $this->prepareEnableModule($this->productionModules);
      $this->disableModules = $this->prepareDisableModule($this->developmentModules);
    }
    return $this;
  }

  /**
   * @inheritDoc
   */
  public function processModules($listOnly) {
    if (!$listOnly) {
      $this
        ->enableModules()
        ->disableModules();
    }
    return array(
      'enable' => $this->enableModules,
      'disable' => $this->disableModules,
    );
  }

  /**
   * Returns default list of development modules.
   *
   * @return array
   */
  protected function developmentDefaults() {
    return array(
      'browser_refresh',
      'coffee',
      'devel',
      'field_ui',
      'hacked',
      'kint',
      'link_css',
      'rules_ui',
      'stage_file_proxy',
      'views_ui',
    );
  }

  /**
   * Returns default list of production modules.
   *
   * @return array
   */
  protected function productionDefaults() {
    return array(
      'advagg',
      'bigpipe',
      'cdn',
      'cloudflare',
      'cloudflarepurger',
      'dynamic_page_cache',
      'page_cache',
      'purge',
      'varnish_purger',
    );
  }

  /**
   * @param string $flag
   *
   * @return bool
   * @throws \Exception
   */
  protected function flag($flag) {
    switch (strtolower($flag)) {
      case 'on':
      case 'yes':
      case 'true':
      case '1':
        return TRUE;
        break;

      case 'off':
      case 'no':
      case 'false':
      case '0':
        return FALSE;
        break;

      default:
        throw new \Exception('Unsupported flag.');
    }
  }

}
